% group = findvalues(v);
%
% Returns several arrays with indices of distinct values
% group(i).indices gives the indices in v with value i
% This is faster than calling find(v == i) several times
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function group = findvalues(v);

group = struct('indices',{});
for i = 1:length(v)
    if ~isnan(v(i))
        if floor(v(i)) ~= v(i)
            warning('findvalues: non-integer value found');
            continue;
        end
        if v(i) < 1
            warning('findvalues: value < 1 found');
            continue;
        end
        if v(i) > length(group)
            group(v(i)).indices = i;
        else
            group(v(i)).indices = [group(v(i)).indices i];
        end
    end
end
