% [test_labels test_values C] = gnb(test_data,train_data,train_labels);
%
% Run Gaussian Naive Bayes classifier
% Learns on train set, predicts test set
%   train_data, test_data: #instances by #features data matrices
%   train_labels:          #instances vector (with values 1 or 2)
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [test_labels test_values C] = gnb(test_data,train_data,train_labels);

% Check input
[num_instances num_features] = size(train_data);
if size(test_data,2) ~= num_features
    error('Training and testing set have different number of features');
end
if length(train_labels) ~= num_instances
    error('Number of labels is different from number of instances in training set');
end

% Learn on training data
idx1 = find(train_labels == 1);
idx2 = find(train_labels == 2);
C.p1 = length(idx1) / num_instances;
C.mu1 = mean(train_data(idx1,:),1);
C.sigma12 = var(train_data(idx1,:),1,1);
C.mu2 = mean(train_data(idx2,:),1);
C.sigma22 = var(train_data(idx2,:),1,1);

% Predict on test data
oo = ones(size(test_data,1),1);
test_values = (-sum(log(C.sigma12)) - sum((test_data - oo*C.mu1).^2 ./ (oo*C.sigma12),2) + 2*log(C.p1)) - ...
              (-sum(log(C.sigma22)) - sum((test_data - oo*C.mu2).^2 ./ (oo*C.sigma22),2) + 2*log(1-C.p1));
test_labels = 2 - double(test_values > 0);
