% test_labels = ridge_fld(test_data,train_data,train_labels,ridge);
%
% Run Fisher linear discriminant
% Learns on train set, predicts test set
%   train_data, test_data: #instances by #features data matrices
%   train_labels:          #instances vector (with values 1 or 2)
%   ridge:                 double
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function test_labels = ridge_fld(test_data,train_data,train_labels,ridge);

num_features = size(train_data,2);
idx1 = find(train_labels == 1);
idx2 = find(train_labels == 2);
mu1 = mean(train_data(idx1,:),1);
data1 = train_data(idx1,:) - repmat(mu1,length(idx1),1);
if ridge == inf
    S1 = sparse(1:num_features,1:num_features,sum(data1.^2,1));
else
    S1 = data1.' * data1 + ridge*eye(num_features);
end
mu2 = mean(train_data(idx2,:),1);
data2 = train_data(idx2,:) - repmat(mu2,length(idx2),1);
if ridge == inf
    S2 = sparse(1:num_features,1:num_features,sum(data2.^2,1));
else
    S2 = data2.' * data2 + ridge*eye(num_features);
end
test_labels = fld(test_data,mu1,S1,mu2,S2);
